%% Salt dome detection using PC
% ===============================================================================
%  Please, cite the following papers in your publication if you use this script.
% ===============================================================================
% M. A. Shafiq, Y. Alaudah, G. AlRegib, and M. Deriche, Phase Congruency for Image understanding with 
% applications in Computational Seismic Interpretation, The 42nd IEEE International Conference on Acoustics, 
% Speech and Signal Processing (ICASSP). The Internet of Signals, New Orleans, USA, March 5-9, 2017.

% If you have found any bugs or have any questions/suggestions, 
% please contact amirshafiq@gatech.edu 

%% Phase Congruency for Salt dome detection
% Init
clc
clear
close all

%% Add path
addpath(genpath('Functions'))
addpath(genpath('Mat Files'))

load SD_3DGoT
load SD_Aqrawi_fixed
load SD_Oslu

%% Load data
load salt2_inline
SS_dataset = salt2_inline;
clear salt2_inline
[nRows,nCols,nSS] = size(SS_dataset);

% Seed points
load iSeed_pts_dataset
capture_SP = 0;

% Frechet Sim
load Frechet_Sim

% Sim Parameter
SS_start = 120;
SS_stop  = 143;
Frech_Calc  = 1;

%% Simulation

for SS_loop = SS_start:SS_stop
    
    SS_num = SS_loop;
    fprintf('Calculating Phase Congruency at Seismic Section Inline #%d... \n',SS_num);
    SS_UT  = SS_dataset(:,:,SS_num);
    
    %% Phase Congruency
    Ph_Cong =  phasecong3(SS_UT,3,4,4,1.6,0.3,2,0.5,10,-1);

    %% Post Processing
    BW_open_param  = 40;
    BW_dilb4_RG    = 6;         
    SD_dilafter_RG = 11;     
    p_time         = 0.5;
    
    Ph_Cong   = imNormalize(Ph_Cong,2);                       
    PC_BinImg = im2bw(Ph_Cong, graythresh(Ph_Cong)-0.01);     
    PC_BinImg = bwareaopen(PC_BinImg, BW_open_param);         
    PC_BinImg = imdilate(PC_BinImg, strel('disk', BW_dilb4_RG));  
    PC_BinImg = imclose(PC_BinImg, strel('disk', BW_dilb4_RG+5)); 

    % Boundary of SS
    SS_val_dummy = 3;                                        % Pixel width
    PC_BinImg(end-SS_val_dummy:end, SS_val_dummy:end)= 1;    % Lower Line
    PC_BinImg(1:end,1:SS_val_dummy)                  = 1;    % Left Line
   
    % Region Growing
    if capture_SP
        [Salt_Dome_SS, iS] = RegionGrowing_mod(PC_BinImg,1);  
    else
        Seed_pt = round(iSeed{SS_num});
        Seed_pt = fliplr(Seed_pt); 
        [Salt_Dome_SS, iS] = RegionGrowing_mod(PC_BinImg,1,Seed_pt);   
    end
    % Post RG Processing
    Salt_Dome_SS  = imdilate(Salt_Dome_SS, strel('disk', SD_dilafter_RG));     % Dilation
    Salt_Dome_SS  = imerode(Salt_Dome_SS, strel('disk', 2));                   % Erosion

    % Salt Dome Boundary
    Salt_Dome_B   = bwperim(Salt_Dome_SS);   % Perimeter of salt dome
    Salt_Dome_B(137,:) = false;

    %% Objective Assessment
    
     if (SS_num==120)||(SS_num==129)||(SS_num==135)||(SS_num==143) 
        % Ground Truth
        read_GT_file = ['Ground_Truth_SS',num2str(SS_num),'.mat'];
        load (read_GT_file,'-mat')
        Ground_TruthDisp    = GT_SS;
        clear GT_SS;
        Oslu_SD   = SD_Oslu(:,:,SS_num);
        Aqrawi_SD = SD_Aqrawif(:,:,SS_num);
        GoT_3D_SD = SD_3DGoT(:,:,SS_num);
        Img_read  = SS_dataset(:,:,SS_num);
        figure
        imshow (Img_read, []); 
        hold on; axis off; 
        contour(GoT_3D_SD,    [0.5 0.5],'b','LineWidth',2);
        contour(Aqrawi_SD,    [0.5 0.5],'m','LineWidth',2);
        contour(Oslu_SD,      [0.5 0.5],'y','LineWidth',2);
        contour(Salt_Dome_SS, [0.5 0.5],'r','LineWidth',2);
        contour(Ground_TruthDisp, [0.5 0.5],'g','LineWidth',2);
        hold off
        title(['Seismic Section ', num2str(SS_num) ], 'FontSize', 24 , 'fontname','Book Antiqua');
    end
    if Frech_Calc
        % Load Ground Truth
        load(['Ground_Truth_SS', num2str(SS_num)],'-mat')
        Ground_Truth  = GT_SS;  
        clear GT_SS;
        Ground_TruthB  = bwperim(Ground_Truth);
        Ground_TruthB(137,:) = false;
        SalSIM_Mod(SS_loop) = SalSIM_cmpr_mod_v01(Ground_TruthB,Salt_Dome_B);
    end
end

%% Display SalSIM Index
if Frech_Calc

    figure
    SS_indexes = 249+(SS_start:SS_stop);
    hold on
    plot(SS_indexes, Frechet_Aqrawif(SS_start:SS_stop),'<-m','linewidth',3,'Markersize',10)
    plot(SS_indexes, Frechet_Oslu(SS_start:SS_stop),'>-y','linewidth',3,'Markersize',10)
    plot(SS_indexes, Frechet_3D_GoT(SS_start:SS_stop),'o-b','linewidth',3,'Markersize',10)
    plot(SS_indexes, SalSIM_Mod(SS_start:SS_stop), '+-r', 'linewidth', 3,'Markersize',10)
    hold off
    axis tight
    l_str = {'Aqrawi et al. [10]','Berthelot et al. [12]','Shafiq et al. [13]','Proposed Method'};
    legend(l_str,'FontSize', 32,  'fontname','Times New roman','Location','southeast')
    xlabel('Seismic section inlines', 'FontSize', 30 , 'fontname','Times New roman')
    ylabel('SalSIM indices', 'FontSize', 30 , 'fontname','Times New roman')
    title('Objective Assesment of Similarity', 'FontSize', 24 , 'fontname','Times New roman');
    box on
    grid on 
    set(gca,'fontsize',30,'fontname','Times New Roman')  % set axis font size
    ylim([0.6 1])
    
    %% Mean
    Mean_Ph_Cong      = mean(SalSIM_Mod(SS_start:SS_stop));
    Mean_3DGoT        = mean(Frechet_3D_GoT(SS_start:SS_stop));
    Mean_Oslu         = mean(Frechet_Oslu(SS_start:SS_stop));
    Mean_Aqrawi 	  = mean(Frechet_Aqrawif(SS_start:SS_stop));

    Std_Ph_Cong       = std(SalSIM_Mod(SS_start:SS_stop));
    Std_3DGoT         = std(Frechet_3D_GoT(SS_start:SS_stop));
    Std_Oslu          = std(Frechet_Oslu(SS_start:SS_stop));
    Std_Aqrawi        = std(Frechet_Aqrawif(SS_start:SS_stop));

    fprintf('\n****************    Frechet Similarity Mean    ****************\n');
    fprintf('Phase Cong  Mean for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Mean_Ph_Cong );
    fprintf('3D GoT      Mean for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Mean_3DGoT );
    fprintf('Oslu        Mean for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Mean_Oslu );
    fprintf('Aqrawi      Mean for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Mean_Aqrawi );

    fprintf('\n****************    Frechet Similarity Std    ****************\n');
    fprintf('Phase Cong  Standard Dev for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Std_Ph_Cong );
    fprintf('3D GoT      Standard Dev for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Std_3DGoT );
    fprintf('Oslu        Standard Dev for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Std_Oslu );
    fprintf('Aqrawi      Standard Dev for SS#%d to SS#%d is ==> %f \n',SS_start,SS_stop, Std_Aqrawi );

end



